--
-- Created by IntelliJ IDEA.
-- User: Chris
-- Date: 24/01/2019
-- Time: 03:36
-- To change this template use File | Settings | File Templates.
--

require '../commands/admin'
require '../commands/team'
require '../commands/coop'
require '../commands/user'
require 'errors'
require 'permissions'
require 'serpent'

command_functions = {}
command_security_level = {}
command_parameters_expectation = {}
--------------------------------------------------------------------------------

-- sec level 0: user, 1: team player, 2: team leader, 3: admin
function add_command_alias(alias, func, sec_level, param_expected)
    command_functions[alias] = func
    command_security_level[alias] = sec_level
    command_parameters_expectation[alias] = param_expected


end

--------------------------------------------------------------------------------
function load_command_aliases()
    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
    add_command_alias("universe-spawn", go_to_universe_spawn, 0, false)
    add_command_alias("universe-set-home", set_universe_home, 0, false)
    add_command_alias("universe-settings", universe_settings, 2, false)
    add_command_alias("universe-level", get_universe_level, 1, false)
    add_command_alias("universe-reset", reset_universe, 1, false)
    add_command_alias("universe-expel", expel_visitor, 1, true)
    add_command_alias("universe-switch", go_to_universe, 0, true)
    add_command_alias("universe-home", go_to_universe_home, 0, false)
    add_command_alias("universe-lock", lock_universe, 1, false)
    add_command_alias("universe-unlock", lock_universe, 1, false)
    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
    add_command_alias("universe-accept", accept_invitation, 0, false)
    add_command_alias("universe-reject", reject_invitation, 0, false)
    add_command_alias("universe-kick", kick_from_team, 2, true)
    add_command_alias("universe-invite", invite_to_team, 1, true)
    add_command_alias("universe-info", team_info, 1, false)
    add_command_alias("universe-leave", leave_team, 1, false)
    add_command_alias("universe-make-leader", make_leader, 1, true)
    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
    add_command_alias("universe-coop", coop, 1, true)
    add_command_alias("universe-expel", expel_from_coop, 1, true)
    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    add_command_alias("universe-delete", delete_universe, 3, true)
    add_command_alias("universe-purge", purge_old_universes, 3, true)
    add_command_alias("universe-top-ten", get_top_ten_universes, 3, false)
    add_command_alias("universe-unregister", unregister_player_from_universe, 3, true)
    add_command_alias("universe-team-remove", remove_player_from_teams, 3, true)
    add_command_alias("universe-team-add", add_player_to_leader_team, 3, true)

    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
end

--------------------------------------------------------------------------------

--data format:
--name :: string: Name of the command.
--tick :: uint: Tick the command was used.
--player_index :: uint: The player who used the command.
--parameter :: string (optional): The parameter passed after the command, separated from the command by 1 space.

function mysplit(inputstr, sep)
    if sep == nil then
        sep = "%s"
    end
    local t = {}; local i = 1
    for str in string.gmatch(inputstr, "([^" .. sep .. "]+)") do
        t[i] = str
        i = i + 1
    end
    return t
end

function parse_parameters(string)
    return mysplit(string)
end

function process_command(data)
    local name = data["name"]
    game.print(serpent.block(data))

    if command_parameters_expectation[name] then
        if data["parameter"] == nil then
            return missingParametersError(name)
        end
        local parameters = parse_parameters(data["parameter"])
    end


    local param_offset = 0
    if data["name"] == "universe" then
        --name = parameters[0]
        param_offset = 1
    end
    local player_id = data["player_index"]

    if command_security_level[name] == 1 then -- team member
        if not has_team(player_id) then
            return permissionDeniedError(player_id, "Not in a team")
        end
    elseif command_security_level[name] == 2 then --team leader
        if not is_team_leader(player_id) then
            return permissionDeniedError(player_id, "Not team leader")
        end
    elseif command_security_level[name] == 3 then --admin
        if not is_admin(player_id) then
            return permissionDeniedError(player_id, "Not an admin")
        end
    end
    game.print(name)
    game.print(serpent.block(data))
    return command_functions[name](data)
end